/****************************************************************************
** Meta object code from reading C++ file 'notifymodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../notifymodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notifymodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12notifycenter11NotifyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto notifycenter::NotifyModel::qt_create_metaobjectdata<qt_meta_tag_ZN12notifycenter11NotifyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "notifycenter::NotifyModel",
        "QML.Element",
        "auto",
        "dataInfoChanged",
        "",
        "collapseChanged",
        "countChanged",
        "doEntityReceived",
        "id",
        "onCountChanged",
        "expandApp",
        "row",
        "collapseApp",
        "remove",
        "removeByApp",
        "appName",
        "clear",
        "collapseAllApp",
        "expandAllApp",
        "open",
        "close",
        "invokeAction",
        "actionId",
        "pinApplication",
        "pin",
        "dataInfo",
        "collapse"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataInfoChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'collapseChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doEntityReceived'
        QtMocHelpers::SlotData<void(qint64)>(7, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 8 },
        }}),
        // Slot 'onCountChanged'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'expandApp'
        QtMocHelpers::MethodData<void(int)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Method 'collapseApp'
        QtMocHelpers::MethodData<void(int)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qint64)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 },
        }}),
        // Method 'removeByApp'
        QtMocHelpers::MethodData<void(const QString &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'collapseAllApp'
        QtMocHelpers::MethodData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'expandAllApp'
        QtMocHelpers::MethodData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'open'
        QtMocHelpers::MethodData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'close'
        QtMocHelpers::MethodData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'invokeAction'
        QtMocHelpers::MethodData<void(qint64, const QString &)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 8 }, { QMetaType::QString, 22 },
        }}),
        // Method 'pinApplication'
        QtMocHelpers::MethodData<void(const QString &, bool)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataInfo'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 0),
        // property 'collapse'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<NotifyModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject notifycenter::NotifyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter11NotifyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter11NotifyModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12notifycenter11NotifyModelE_t>.metaTypes,
    nullptr
} };

void notifycenter::NotifyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NotifyModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataInfoChanged(); break;
        case 1: _t->collapseChanged(); break;
        case 2: _t->countChanged(); break;
        case 3: _t->doEntityReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 4: _t->onCountChanged(); break;
        case 5: _t->expandApp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->collapseApp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->remove((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 8: _t->removeByApp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->clear(); break;
        case 10: _t->collapseAllApp(); break;
        case 11: _t->expandAllApp(); break;
        case 12: _t->open(); break;
        case 13: _t->close(); break;
        case 14: _t->invokeAction((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->pinApplication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NotifyModel::*)()>(_a, &NotifyModel::dataInfoChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyModel::*)()>(_a, &NotifyModel::collapseChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyModel::*)()>(_a, &NotifyModel::countChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->dataInfo(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->collapse(); break;
        default: break;
        }
    }
}

const QMetaObject *notifycenter::NotifyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *notifycenter::NotifyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter11NotifyModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int notifycenter::NotifyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void notifycenter::NotifyModel::dataInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void notifycenter::NotifyModel::collapseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void notifycenter::NotifyModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
