/****************************************************************************
** Meta object code from reading C++ file 'dockpanel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dockpanel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dockpanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock9DockPanelE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::DockPanel::qt_create_metaobjectdata<qt_meta_tag_ZN4dock9DockPanelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::DockPanel",
        "geometryChanged",
        "",
        "QRect",
        "geometry",
        "frontendWindowRectChanged",
        "frontendWindowRect",
        "hideStateChanged",
        "HideState",
        "state",
        "colorThemeChanged",
        "ColorTheme",
        "theme",
        "compositorReadyChanged",
        "dockSizeChanged",
        "size",
        "hideModeChanged",
        "HideMode",
        "mode",
        "beforePositionChanged",
        "Position",
        "beforePosition",
        "positionChanged",
        "position",
        "itemAlignmentChanged",
        "ItemAlignment",
        "alignment",
        "indicatorStyleChanged",
        "IndicatorStyle",
        "style",
        "showInPrimaryChanged",
        "showInPrimary",
        "dockScreenChanged",
        "QScreen*",
        "screen",
        "screenNameChanged",
        "requestClosePopup",
        "leftEdgeClicked",
        "minOrder",
        "devicePixelRatioChanged",
        "ratio",
        "lockedChanged",
        "locked",
        "contextMenuEnabledChanged",
        "enabled",
        "contextDraggingChanged",
        "isResizingChanged",
        "isResizing",
        "xembedWindowMoveResult",
        "uint32_t",
        "wid",
        "success",
        "onWindowGeometryChanged",
        "launcherVisibleChanged",
        "visible",
        "updateDockScreen",
        "openDockSettings",
        "notifyDockPositionChanged",
        "offsetX",
        "offsetY",
        "moveXEmbedWindow",
        "dx",
        "dy",
        "QQuickWindow*",
        "anchorWindow",
        "compositorReady",
        "hideState",
        "colorTheme",
        "dockSize",
        "hideMode",
        "itemAlignment",
        "indicatorStyle",
        "screenName",
        "contextMenuEnabled",
        "devicePixelRatio",
        "debugMode",
        "contextDragging"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void(QRect)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'frontendWindowRectChanged'
        QtMocHelpers::SignalData<void(QRect)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'hideStateChanged'
        QtMocHelpers::SignalData<void(HideState)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'colorThemeChanged'
        QtMocHelpers::SignalData<void(ColorTheme)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'compositorReadyChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dockSizeChanged'
        QtMocHelpers::SignalData<void(uint)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Signal 'hideModeChanged'
        QtMocHelpers::SignalData<void(HideMode)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'beforePositionChanged'
        QtMocHelpers::SignalData<void(Position)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(Position)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 23 },
        }}),
        // Signal 'itemAlignmentChanged'
        QtMocHelpers::SignalData<void(ItemAlignment)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'indicatorStyleChanged'
        QtMocHelpers::SignalData<void(IndicatorStyle)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'showInPrimaryChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Signal 'dockScreenChanged'
        QtMocHelpers::SignalData<void(QScreen *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Signal 'screenNameChanged'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestClosePopup'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftEdgeClicked'
        QtMocHelpers::SignalData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Signal 'devicePixelRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Signal 'lockedChanged'
        QtMocHelpers::SignalData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 42 },
        }}),
        // Signal 'contextMenuEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
        // Signal 'contextDraggingChanged'
        QtMocHelpers::SignalData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isResizingChanged'
        QtMocHelpers::SignalData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
        // Signal 'xembedWindowMoveResult'
        QtMocHelpers::SignalData<void(uint32_t, bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 49, 50 }, { QMetaType::Bool, 51 },
        }}),
        // Slot 'onWindowGeometryChanged'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'launcherVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 54 },
        }}),
        // Slot 'updateDockScreen'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'openDockSettings'
        QtMocHelpers::MethodData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'notifyDockPositionChanged'
        QtMocHelpers::MethodData<void(int, int)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 58 }, { QMetaType::Int, 59 },
        }}),
        // Method 'moveXEmbedWindow'
        QtMocHelpers::MethodData<bool(uint32_t, double, double, QQuickWindow *)>(60, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 49, 50 }, { QMetaType::Double, 61 }, { QMetaType::Double, 62 }, { 0x80000000 | 63, 64 },
        }}),
        // Method 'moveXEmbedWindow'
        QtMocHelpers::MethodData<bool(uint32_t, double, double)>(60, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 49, 50 }, { QMetaType::Double, 61 }, { QMetaType::Double, 62 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'geometry'
        QtMocHelpers::PropertyData<QRect>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final),
        // property 'frontendWindowRect'
        QtMocHelpers::PropertyData<QRect>(6, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 1),
        // property 'compositorReady'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'hideState'
        QtMocHelpers::PropertyData<HideState>(66, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'colorTheme'
        QtMocHelpers::PropertyData<ColorTheme>(67, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'dockSize'
        QtMocHelpers::PropertyData<uint>(68, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'hideMode'
        QtMocHelpers::PropertyData<HideMode>(69, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'position'
        QtMocHelpers::PropertyData<Position>(23, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
        // property 'itemAlignment'
        QtMocHelpers::PropertyData<ItemAlignment>(70, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 9),
        // property 'indicatorStyle'
        QtMocHelpers::PropertyData<IndicatorStyle>(71, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 10),
        // property 'showInPrimary'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'screenName'
        QtMocHelpers::PropertyData<QString>(72, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 13),
        // property 'locked'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 17),
        // property 'isResizing'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 20),
        // property 'contextMenuEnabled'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 18),
        // property 'devicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(74, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 16),
        // property 'debugMode'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'contextDragging'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 19),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DockPanel, qt_meta_tag_ZN4dock9DockPanelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::DockPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<ds::DPanel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock9DockPanelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock9DockPanelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock9DockPanelE_t>.metaTypes,
    nullptr
} };

void dock::DockPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockPanel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->geometryChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 1: _t->frontendWindowRectChanged((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 2: _t->hideStateChanged((*reinterpret_cast<std::add_pointer_t<HideState>>(_a[1]))); break;
        case 3: _t->colorThemeChanged((*reinterpret_cast<std::add_pointer_t<ColorTheme>>(_a[1]))); break;
        case 4: _t->compositorReadyChanged(); break;
        case 5: _t->dockSizeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 6: _t->hideModeChanged((*reinterpret_cast<std::add_pointer_t<HideMode>>(_a[1]))); break;
        case 7: _t->beforePositionChanged((*reinterpret_cast<std::add_pointer_t<Position>>(_a[1]))); break;
        case 8: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<Position>>(_a[1]))); break;
        case 9: _t->itemAlignmentChanged((*reinterpret_cast<std::add_pointer_t<ItemAlignment>>(_a[1]))); break;
        case 10: _t->indicatorStyleChanged((*reinterpret_cast<std::add_pointer_t<IndicatorStyle>>(_a[1]))); break;
        case 11: _t->showInPrimaryChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->dockScreenChanged((*reinterpret_cast<std::add_pointer_t<QScreen*>>(_a[1]))); break;
        case 13: _t->screenNameChanged(); break;
        case 14: _t->requestClosePopup(); break;
        case 15: _t->leftEdgeClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->devicePixelRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->lockedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->contextMenuEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->contextDraggingChanged(); break;
        case 20: _t->isResizingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->xembedWindowMoveResult((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->onWindowGeometryChanged(); break;
        case 23: _t->launcherVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->updateDockScreen(); break;
        case 25: _t->openDockSettings(); break;
        case 26: _t->notifyDockPositionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 27: { bool _r = _t->moveXEmbedWindow((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QQuickWindow*>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->moveXEmbedWindow((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QScreen* >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickWindow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(QRect )>(_a, &DockPanel::geometryChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(QRect )>(_a, &DockPanel::frontendWindowRectChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(HideState )>(_a, &DockPanel::hideStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(ColorTheme )>(_a, &DockPanel::colorThemeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)()>(_a, &DockPanel::compositorReadyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(uint )>(_a, &DockPanel::dockSizeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(HideMode )>(_a, &DockPanel::hideModeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(Position )>(_a, &DockPanel::beforePositionChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(Position )>(_a, &DockPanel::positionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(ItemAlignment )>(_a, &DockPanel::itemAlignmentChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(IndicatorStyle )>(_a, &DockPanel::indicatorStyleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(bool )>(_a, &DockPanel::showInPrimaryChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(QScreen * )>(_a, &DockPanel::dockScreenChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)()>(_a, &DockPanel::screenNameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)()>(_a, &DockPanel::requestClosePopup, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(const QString & )>(_a, &DockPanel::leftEdgeClicked, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(qreal )>(_a, &DockPanel::devicePixelRatioChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(bool )>(_a, &DockPanel::lockedChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(bool )>(_a, &DockPanel::contextMenuEnabledChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)()>(_a, &DockPanel::contextDraggingChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(bool )>(_a, &DockPanel::isResizingChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanel::*)(uint32_t , bool )>(_a, &DockPanel::xembedWindowMoveResult, 21))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QRect*>(_v) = _t->geometry(); break;
        case 1: *reinterpret_cast<QRect*>(_v) = _t->frontendWindowRect(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->compositorReady(); break;
        case 3: *reinterpret_cast<HideState*>(_v) = _t->hideState(); break;
        case 4: *reinterpret_cast<ColorTheme*>(_v) = _t->colorTheme(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->dockSize(); break;
        case 6: *reinterpret_cast<HideMode*>(_v) = _t->hideMode(); break;
        case 7: *reinterpret_cast<Position*>(_v) = _t->position(); break;
        case 8: *reinterpret_cast<ItemAlignment*>(_v) = _t->itemAlignment(); break;
        case 9: *reinterpret_cast<IndicatorStyle*>(_v) = _t->indicatorStyle(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->showInPrimary(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->screenName(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->locked(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isResizing(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->contextMenuEnabled(); break;
        case 15: *reinterpret_cast<qreal*>(_v) = _t->devicePixelRatio(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->debugMode(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->contextDragging(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setCompositorReady(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setHideState(*reinterpret_cast<HideState*>(_v)); break;
        case 4: _t->setColorTheme(*reinterpret_cast<ColorTheme*>(_v)); break;
        case 5: _t->setDockSize(*reinterpret_cast<uint*>(_v)); break;
        case 6: _t->setHideMode(*reinterpret_cast<HideMode*>(_v)); break;
        case 7: _t->setPosition(*reinterpret_cast<Position*>(_v)); break;
        case 8: _t->setItemAlignment(*reinterpret_cast<ItemAlignment*>(_v)); break;
        case 9: _t->setIndicatorStyle(*reinterpret_cast<IndicatorStyle*>(_v)); break;
        case 10: _t->setShowInPrimary(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setLocked(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setIsResizing(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setContextDragging(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dock::DockPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::DockPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock9DockPanelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return ds::DPanel::qt_metacast(_clname);
}

int dock::DockPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ds::DPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void dock::DockPanel::geometryChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void dock::DockPanel::frontendWindowRectChanged(QRect _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void dock::DockPanel::hideStateChanged(HideState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dock::DockPanel::colorThemeChanged(ColorTheme _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void dock::DockPanel::compositorReadyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dock::DockPanel::dockSizeChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void dock::DockPanel::hideModeChanged(HideMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void dock::DockPanel::beforePositionChanged(Position _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void dock::DockPanel::positionChanged(Position _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void dock::DockPanel::itemAlignmentChanged(ItemAlignment _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void dock::DockPanel::indicatorStyleChanged(IndicatorStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void dock::DockPanel::showInPrimaryChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void dock::DockPanel::dockScreenChanged(QScreen * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void dock::DockPanel::screenNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void dock::DockPanel::requestClosePopup()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void dock::DockPanel::leftEdgeClicked(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void dock::DockPanel::devicePixelRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void dock::DockPanel::lockedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void dock::DockPanel::contextMenuEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void dock::DockPanel::contextDraggingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void dock::DockPanel::isResizingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void dock::DockPanel::xembedWindowMoveResult(uint32_t _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}
QT_WARNING_POP
