/****************************************************************************
** Meta object code from reading C++ file 'trayitempositionmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../trayitempositionmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'trayitempositionmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8docktray9DropIndexE_t {};
} // unnamed namespace

template <> constexpr inline auto docktray::DropIndex::qt_create_metaobjectdata<qt_meta_tag_ZN8docktray9DropIndexE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "docktray::DropIndex",
        "QML.Element",
        "auto",
        "index",
        "isOnItem",
        "isBefore"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'index'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'isOnItem'
        QtMocHelpers::PropertyData<bool>(4, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'isBefore'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<DropIndex, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject docktray::DropIndex::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8docktray9DropIndexE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8docktray9DropIndexE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8docktray9DropIndexE_t>.metaTypes,
    nullptr
} };

void docktray::DropIndex::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<DropIndex *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->index; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isOnItem; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isBefore; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->index, *reinterpret_cast<int*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->isOnItem, *reinterpret_cast<bool*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->isBefore, *reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto docktray::TrayItemPositionManager::qt_create_metaobjectdata<qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "docktray::TrayItemPositionManager",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "orientationChanged",
        "",
        "Qt::Orientation",
        "dockHeightChanged",
        "visualSizeChanged",
        "QSize",
        "visualItemCountChanged",
        "visualItemSizeChanged",
        "itemIndexByPoint",
        "DropIndex",
        "QPoint",
        "point",
        "layoutHealthCheck",
        "delayMs",
        "clearRegisteredSizes",
        "orientation",
        "dockHeight",
        "visualSize",
        "visualItemCount",
        "itemVisualSize",
        "itemSpacing",
        "itemPadding"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'orientationChanged'
        QtMocHelpers::SignalData<void(Qt::Orientation)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 6 },
        }}),
        // Signal 'dockHeightChanged'
        QtMocHelpers::SignalData<void(int)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'visualSizeChanged'
        QtMocHelpers::SignalData<void(QSize)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 6 },
        }}),
        // Signal 'visualItemCountChanged'
        QtMocHelpers::SignalData<void(int)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'visualItemSizeChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'itemIndexByPoint'
        QtMocHelpers::MethodData<DropIndex(const QPoint) const>(13, 6, QMC::AccessPublic, 0x80000000 | 14, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Method 'layoutHealthCheck'
        QtMocHelpers::MethodData<void(int)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Method 'layoutHealthCheck'
        QtMocHelpers::MethodData<void()>(17, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'clearRegisteredSizes'
        QtMocHelpers::MethodData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'orientation'
        QtMocHelpers::PropertyData<Qt::Orientation>(20, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 0),
        // property 'dockHeight'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'visualSize'
        QtMocHelpers::PropertyData<QSize>(22, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'visualItemCount'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'itemVisualSize'
        QtMocHelpers::PropertyData<QSize>(24, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'itemSpacing'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'itemPadding'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<TrayItemPositionManager, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject docktray::TrayItemPositionManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t>.metaTypes,
    nullptr
} };

void docktray::TrayItemPositionManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TrayItemPositionManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->orientationChanged((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1]))); break;
        case 1: _t->dockHeightChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->visualSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 3: _t->visualItemCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->visualItemSizeChanged(); break;
        case 5: { DropIndex _r = _t->itemIndexByPoint((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DropIndex*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->layoutHealthCheck((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->layoutHealthCheck(); break;
        case 8: _t->clearRegisteredSizes(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TrayItemPositionManager::*)(Qt::Orientation )>(_a, &TrayItemPositionManager::orientationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItemPositionManager::*)(int )>(_a, &TrayItemPositionManager::dockHeightChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItemPositionManager::*)(QSize )>(_a, &TrayItemPositionManager::visualSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItemPositionManager::*)(int )>(_a, &TrayItemPositionManager::visualItemCountChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItemPositionManager::*)()>(_a, &TrayItemPositionManager::visualItemSizeChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Qt::Orientation*>(_v) = _t->m_orientation; break;
        case 1: *reinterpret_cast<int*>(_v) = _t->m_dockHeight; break;
        case 2: *reinterpret_cast<QSize*>(_v) = _t->m_visualSize; break;
        case 3: *reinterpret_cast<int*>(_v) = _t->m_visualItemCount; break;
        case 4: *reinterpret_cast<QSize*>(_v) = _t->m_itemVisualSize; break;
        case 5: *reinterpret_cast<int*>(_v) = _t->m_itemSpacing; break;
        case 6: *reinterpret_cast<int*>(_v) = _t->m_itemPadding; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0:
            if (QtMocHelpers::setProperty(_t->m_orientation, *reinterpret_cast<Qt::Orientation*>(_v)))
                Q_EMIT _t->orientationChanged(_t->m_orientation);
            break;
        case 1:
            if (QtMocHelpers::setProperty(_t->m_dockHeight, *reinterpret_cast<int*>(_v)))
                Q_EMIT _t->dockHeightChanged(_t->m_dockHeight);
            break;
        case 2:
            if (QtMocHelpers::setProperty(_t->m_visualSize, *reinterpret_cast<QSize*>(_v)))
                Q_EMIT _t->visualSizeChanged(_t->m_visualSize);
            break;
        case 3:
            if (QtMocHelpers::setProperty(_t->m_visualItemCount, *reinterpret_cast<int*>(_v)))
                Q_EMIT _t->visualItemCountChanged(_t->m_visualItemCount);
            break;
        default: break;
        }
    }
}

const QMetaObject *docktray::TrayItemPositionManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *docktray::TrayItemPositionManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8docktray23TrayItemPositionManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int docktray::TrayItemPositionManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void docktray::TrayItemPositionManager::orientationChanged(Qt::Orientation _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void docktray::TrayItemPositionManager::dockHeightChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void docktray::TrayItemPositionManager::visualSizeChanged(QSize _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void docktray::TrayItemPositionManager::visualItemCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void docktray::TrayItemPositionManager::visualItemSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
