/****************************************************************************
** Meta object code from reading C++ file 'appitemmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../appitemmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appitemmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4apps12AppItemModelE_t {};
} // unnamed namespace

template <> constexpr inline auto apps::AppItemModel::qt_create_metaobjectdata<qt_meta_tag_ZN4apps12AppItemModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "apps::AppItemModel",
        "Roles",
        "DesktopIdRole",
        "NameRole",
        "IconNameRole",
        "StartUpWMClassRole",
        "NoDisplayRole",
        "ActionsRole",
        "DDECategoryRole",
        "InstalledTimeRole",
        "LastLaunchedTimeRole",
        "LaunchedTimesRole",
        "DockedRole",
        "OnDesktopRole",
        "AutoStartRole",
        "AppTypeRole",
        "XLingLongRole",
        "IdRole",
        "XCreatedByRole",
        "ExecsRole",
        "CategoriesRole",
        "DesktopSourcePathRole",
        "VendorRole",
        "GenericNameRole",
        "AppTypes",
        "AppItemType",
        "FolderItemType",
        "DDECategories",
        "Internet",
        "Chat",
        "Music",
        "Video",
        "Graphics",
        "Game",
        "Office",
        "Reading",
        "Development",
        "System",
        "Others"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(1, 1, QMC::EnumFlags{}).add({
            {    2, Roles::DesktopIdRole },
            {    3, Roles::NameRole },
            {    4, Roles::IconNameRole },
            {    5, Roles::StartUpWMClassRole },
            {    6, Roles::NoDisplayRole },
            {    7, Roles::ActionsRole },
            {    8, Roles::DDECategoryRole },
            {    9, Roles::InstalledTimeRole },
            {   10, Roles::LastLaunchedTimeRole },
            {   11, Roles::LaunchedTimesRole },
            {   12, Roles::DockedRole },
            {   13, Roles::OnDesktopRole },
            {   14, Roles::AutoStartRole },
            {   15, Roles::AppTypeRole },
            {   16, Roles::XLingLongRole },
            {   17, Roles::IdRole },
            {   18, Roles::XCreatedByRole },
            {   19, Roles::ExecsRole },
            {   20, Roles::CategoriesRole },
            {   21, Roles::DesktopSourcePathRole },
            {   22, Roles::VendorRole },
            {   23, Roles::GenericNameRole },
        }),
        // enum 'AppTypes'
        QtMocHelpers::EnumData<enum AppTypes>(24, 24, QMC::EnumFlags{}).add({
            {   25, AppTypes::AppItemType },
            {   26, AppTypes::FolderItemType },
        }),
        // enum 'DDECategories'
        QtMocHelpers::EnumData<enum DDECategories>(27, 27, QMC::EnumFlags{}).add({
            {   28, DDECategories::Internet },
            {   29, DDECategories::Chat },
            {   30, DDECategories::Music },
            {   31, DDECategories::Video },
            {   32, DDECategories::Graphics },
            {   33, DDECategories::Game },
            {   34, DDECategories::Office },
            {   35, DDECategories::Reading },
            {   36, DDECategories::Development },
            {   37, DDECategories::System },
            {   38, DDECategories::Others },
        }),
    };
    return QtMocHelpers::metaObjectData<AppItemModel, qt_meta_tag_ZN4apps12AppItemModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject apps::AppItemModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStandardItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4apps12AppItemModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4apps12AppItemModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4apps12AppItemModelE_t>.metaTypes,
    nullptr
} };

void apps::AppItemModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppItemModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *apps::AppItemModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *apps::AppItemModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4apps12AppItemModelE_t>.strings))
        return static_cast<void*>(this);
    return QStandardItemModel::qt_metacast(_clname);
}

int apps::AppItemModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStandardItemModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
