/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"


static const struct wl_interface *treeland_wallpaper_color_v1_types[] = {
	NULL,
	NULL,
};

static const struct wl_message treeland_wallpaper_color_manager_v1_requests[] = {
	{ "watch", "s", treeland_wallpaper_color_v1_types + 0 },
	{ "unwatch", "s", treeland_wallpaper_color_v1_types + 0 },
	{ "destroy", "", treeland_wallpaper_color_v1_types + 0 },
};

static const struct wl_message treeland_wallpaper_color_manager_v1_events[] = {
	{ "output_color", "su", treeland_wallpaper_color_v1_types + 0 },
};

WL_EXPORT const struct wl_interface treeland_wallpaper_color_manager_v1_interface = {
	"treeland_wallpaper_color_manager_v1", 1,
	3, treeland_wallpaper_color_manager_v1_requests,
	1, treeland_wallpaper_color_manager_v1_events,
};

