/****************************************************************************
** Meta object code from reading C++ file 'taskmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../taskmanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taskmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock11TaskManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TaskManager::qt_create_metaobjectdata<qt_meta_tag_ZN4dock11TaskManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TaskManager",
        "dataModelChanged",
        "",
        "windowSplitChanged",
        "windowFullscreenChanged",
        "allowedForceQuitChanged",
        "showAttentionAnimationChanged",
        "handleWindowAdded",
        "QPointer<AbstractWindow>",
        "window",
        "modifyOpacityChanged",
        "requestActivate",
        "QModelIndex",
        "index",
        "requestNewInstance",
        "action",
        "requestOpenUrls",
        "QList<QUrl>",
        "urls",
        "requestClose",
        "force",
        "requestUpdateWindowIconGeometry",
        "QRect",
        "geometry",
        "delegate",
        "requestPreview",
        "relativePositionItem",
        "int32_t",
        "previewXoffset",
        "previewYoffset",
        "uint32_t",
        "direction",
        "requestWindowsView",
        "QModelIndexList",
        "indexes",
        "desktopIdToAppId",
        "desktopId",
        "requestDockByDesktopId",
        "desktopID",
        "requestUndockByDesktopId",
        "RequestDock",
        "appID",
        "IsDocked",
        "RequestUndock",
        "dropFilesOnItem",
        "itemId",
        "hideItemPreview",
        "activateWindow",
        "windowID",
        "saveDockElementsOrder",
        "appIds",
        "moveItem",
        "from",
        "to",
        "getTrashTipText",
        "isTrashEmpty",
        "dataModel",
        "QAbstractItemModel*",
        "windowSplit",
        "windowFullscreen",
        "allowForceQuit",
        "showAttentionAnimation",
        "Roles",
        "WinIdRole",
        "PidRole",
        "IdentityRole",
        "WinIconRole",
        "WinTitleRole",
        "ActiveRole",
        "ShouldSkipRole",
        "AttentionRole",
        "ItemIdRole",
        "MenusRole",
        "WindowsRole",
        "DesktopIdRole",
        "NameRole",
        "IconNameRole",
        "StartUpWMClassRole",
        "NoDisplayRole",
        "ActionsRole",
        "DDECategoryRole",
        "InstalledTimeRole",
        "LastLaunchedTimeRole",
        "LaunchedTimesRole",
        "DockedRole",
        "OnDesktopRole",
        "AutoStartRole",
        "AppTypeRole",
        "XLingLongRole",
        "IdRole",
        "XCreatedByRole",
        "ExecsRole",
        "CategoriesRole",
        "DesktopSourcePathRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataModelChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowSplitChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowFullscreenChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'allowedForceQuitChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showAttentionAnimationChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleWindowAdded'
        QtMocHelpers::SlotData<void(QPointer<AbstractWindow>)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'modifyOpacityChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'requestActivate'
        QtMocHelpers::MethodData<void(const QModelIndex &) const>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'requestNewInstance'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString &) const>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::QString, 15 },
        }}),
        // Method 'requestNewInstance'
        QtMocHelpers::MethodData<void(const QModelIndex &) const>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'requestOpenUrls'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QList<QUrl> &) const>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 17, 18 },
        }}),
        // Method 'requestClose'
        QtMocHelpers::MethodData<void(const QModelIndex &, bool) const>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 20 },
        }}),
        // Method 'requestClose'
        QtMocHelpers::MethodData<void(const QModelIndex &) const>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'requestUpdateWindowIconGeometry'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QRect &, QObject *) const>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 22, 23 }, { QMetaType::QObjectStar, 24 },
        }}),
        // Method 'requestUpdateWindowIconGeometry'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QRect &) const>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 22, 23 },
        }}),
        // Method 'requestPreview'
        QtMocHelpers::MethodData<void(const QModelIndex &, QObject *, int32_t, int32_t, uint32_t)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::QObjectStar, 26 }, { 0x80000000 | 27, 28 }, { 0x80000000 | 27, 29 },
            { 0x80000000 | 30, 31 },
        }}),
        // Method 'requestWindowsView'
        QtMocHelpers::MethodData<void(const QModelIndexList &) const>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Method 'desktopIdToAppId'
        QtMocHelpers::MethodData<QString(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 36 },
        }}),
        // Method 'requestDockByDesktopId'
        QtMocHelpers::MethodData<bool(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 38 },
        }}),
        // Method 'requestUndockByDesktopId'
        QtMocHelpers::MethodData<bool(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 38 },
        }}),
        // Method 'RequestDock'
        QtMocHelpers::MethodData<bool(QString)>(40, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 41 },
        }}),
        // Method 'IsDocked'
        QtMocHelpers::MethodData<bool(QString)>(42, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 41 },
        }}),
        // Method 'RequestUndock'
        QtMocHelpers::MethodData<bool(QString)>(43, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 41 },
        }}),
        // Method 'dropFilesOnItem'
        QtMocHelpers::MethodData<void(const QString &, const QStringList &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 }, { QMetaType::QStringList, 18 },
        }}),
        // Method 'hideItemPreview'
        QtMocHelpers::MethodData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'activateWindow'
        QtMocHelpers::MethodData<void(uint32_t)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 48 },
        }}),
        // Method 'saveDockElementsOrder'
        QtMocHelpers::MethodData<void(const QStringList &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 50 },
        }}),
        // Method 'moveItem'
        QtMocHelpers::MethodData<void(int, int)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 52 }, { QMetaType::Int, 53 },
        }}),
        // Method 'getTrashTipText'
        QtMocHelpers::MethodData<QString()>(54, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'isTrashEmpty'
        QtMocHelpers::MethodData<bool() const>(55, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(56, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'windowSplit'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'windowFullscreen'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'allowForceQuit'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'showAttentionAnimation'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(62, 62, QMC::EnumFlags{}).add({
            {   63, Roles::WinIdRole },
            {   64, Roles::PidRole },
            {   65, Roles::IdentityRole },
            {   66, Roles::WinIconRole },
            {   67, Roles::WinTitleRole },
            {   68, Roles::ActiveRole },
            {   69, Roles::ShouldSkipRole },
            {   70, Roles::AttentionRole },
            {   71, Roles::ItemIdRole },
            {   72, Roles::MenusRole },
            {   73, Roles::WindowsRole },
            {   74, Roles::DesktopIdRole },
            {   75, Roles::NameRole },
            {   76, Roles::IconNameRole },
            {   77, Roles::StartUpWMClassRole },
            {   78, Roles::NoDisplayRole },
            {   79, Roles::ActionsRole },
            {   80, Roles::DDECategoryRole },
            {   81, Roles::InstalledTimeRole },
            {   82, Roles::LastLaunchedTimeRole },
            {   83, Roles::LaunchedTimesRole },
            {   84, Roles::DockedRole },
            {   85, Roles::OnDesktopRole },
            {   86, Roles::AutoStartRole },
            {   87, Roles::AppTypeRole },
            {   88, Roles::XLingLongRole },
            {   89, Roles::IdRole },
            {   90, Roles::XCreatedByRole },
            {   91, Roles::ExecsRole },
            {   92, Roles::CategoriesRole },
            {   93, Roles::DesktopSourcePathRole },
        }),
    };
    return QtMocHelpers::metaObjectData<TaskManager, qt_meta_tag_ZN4dock11TaskManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::TaskManager::staticMetaObject = { {
    QMetaObject::SuperData::link<ds::DContainment::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock11TaskManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock11TaskManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock11TaskManagerE_t>.metaTypes,
    nullptr
} };

void dock::TaskManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaskManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataModelChanged(); break;
        case 1: _t->windowSplitChanged(); break;
        case 2: _t->windowFullscreenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->allowedForceQuitChanged(); break;
        case 4: _t->showAttentionAnimationChanged(); break;
        case 5: _t->handleWindowAdded((*reinterpret_cast<std::add_pointer_t<QPointer<AbstractWindow>>>(_a[1]))); break;
        case 6: _t->modifyOpacityChanged(); break;
        case 7: _t->requestActivate((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->requestNewInstance((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->requestNewInstance((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->requestOpenUrls((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2]))); break;
        case 11: _t->requestClose((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->requestClose((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 13: _t->requestUpdateWindowIconGeometry((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRect>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[3]))); break;
        case 14: _t->requestUpdateWindowIconGeometry((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRect>>(_a[2]))); break;
        case 15: _t->requestPreview((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int32_t>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int32_t>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[5]))); break;
        case 16: _t->requestWindowsView((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 17: { QString _r = _t->desktopIdToAppId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->requestDockByDesktopId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->requestUndockByDesktopId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->RequestDock((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->IsDocked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->RequestUndock((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->dropFilesOnItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 24: _t->hideItemPreview(); break;
        case 25: _t->activateWindow((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 26: _t->saveDockElementsOrder((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 27: _t->moveItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 28: { QString _r = _t->getTrashTipText();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->isTrashEmpty();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPointer<AbstractWindow> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaskManager::*)()>(_a, &TaskManager::dataModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskManager::*)()>(_a, &TaskManager::windowSplitChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskManager::*)(bool )>(_a, &TaskManager::windowFullscreenChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskManager::*)()>(_a, &TaskManager::allowedForceQuitChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaskManager::*)()>(_a, &TaskManager::showAttentionAnimationChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->dataModel(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->windowSplit(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->windowFullscreen(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->allowForceQuit(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->showAttentionAnimation(); break;
        default: break;
        }
    }
}

const QMetaObject *dock::TaskManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TaskManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock11TaskManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractTaskManagerInterface"))
        return static_cast< AbstractTaskManagerInterface*>(this);
    return ds::DContainment::qt_metacast(_clname);
}

int dock::TaskManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ds::DContainment::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dock::TaskManager::dataModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::TaskManager::windowSplitChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::TaskManager::windowFullscreenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dock::TaskManager::allowedForceQuitChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dock::TaskManager::showAttentionAnimationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
