// This file was generated by qtwaylandscanner
// source file is treeland-window-management-v1.xml

#ifndef QT_WAYLAND_TREELAND_WINDOW_MANAGEMENT_V1
#define QT_WAYLAND_TREELAND_WINDOW_MANAGEMENT_V1

#include "wayland-treeland-window-management-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  treeland_window_management_v1
    {
    public:
        treeland_window_management_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_window_management_v1(struct ::treeland_window_management_v1 *object);
        treeland_window_management_v1();

        virtual ~treeland_window_management_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_window_management_v1 *object);

        struct ::treeland_window_management_v1 *object() { return m_treeland_window_management_v1; }
        const struct ::treeland_window_management_v1 *object() const { return m_treeland_window_management_v1; }
        static treeland_window_management_v1 *fromObject(struct ::treeland_window_management_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum desktop_state {
            desktop_state_normal = 0,
            desktop_state_show = 1,
            desktop_state_preview_show = 2,
        };

        void set_desktop(uint32_t state);
        void destroy();

    protected:
        virtual void treeland_window_management_v1_show_desktop(uint32_t state);

    private:
        void init_listener();
        static const struct treeland_window_management_v1_listener m_treeland_window_management_v1_listener;
        static void handle_show_desktop(
            void *data,
            struct ::treeland_window_management_v1 *object,
            uint32_t state);
        struct ::treeland_window_management_v1 *m_treeland_window_management_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
