/****************************************************************************
** Meta object code from reading C++ file 'trayitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../trayitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'trayitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock8TrayItemE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TrayItem::qt_create_metaobjectdata<qt_meta_tag_ZN4dock8TrayItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TrayItem",
        "trayPluginModelChanged",
        "",
        "quickPluginModelChanged",
        "fixedPluginModelChanged",
        "pluginsChanged",
        "dockItemInfos",
        "DockItemInfos",
        "setItemOnDock",
        "settingKey",
        "itemKey",
        "visible",
        "emitPluginsChanged",
        "trayPluginModel",
        "QAbstractItemModel*",
        "quickPluginModel",
        "fixedPluginModel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trayPluginModelChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'quickPluginModelChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fixedPluginModelChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pluginsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'dockItemInfos'
        QtMocHelpers::MethodData<DockItemInfos()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Method 'setItemOnDock'
        QtMocHelpers::MethodData<void(const QString &, const QString &, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Method 'emitPluginsChanged'
        QtMocHelpers::MethodData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'trayPluginModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'quickPluginModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'fixedPluginModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(16, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TrayItem, qt_meta_tag_ZN4dock8TrayItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::TrayItem::staticMetaObject = { {
    QMetaObject::SuperData::link<ds::DApplet::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock8TrayItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock8TrayItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock8TrayItemE_t>.metaTypes,
    nullptr
} };

void dock::TrayItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TrayItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trayPluginModelChanged(); break;
        case 1: _t->quickPluginModelChanged(); break;
        case 2: _t->fixedPluginModelChanged(); break;
        case 3: _t->pluginsChanged(); break;
        case 4: { DockItemInfos _r = _t->dockItemInfos();
            if (_a[0]) *reinterpret_cast<DockItemInfos*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setItemOnDock((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->emitPluginsChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TrayItem::*)()>(_a, &TrayItem::trayPluginModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItem::*)()>(_a, &TrayItem::quickPluginModelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItem::*)()>(_a, &TrayItem::fixedPluginModelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrayItem::*)()>(_a, &TrayItem::pluginsChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->trayPluginModel(); break;
        case 1: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->quickPluginModel(); break;
        case 2: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->fixedPluginModel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTrayPluginModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 1: _t->setQuickPluginModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 2: _t->setFixedPluginModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dock::TrayItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TrayItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock8TrayItemE_t>.strings))
        return static_cast<void*>(this);
    return ds::DApplet::qt_metacast(_clname);
}

int dock::TrayItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ds::DApplet::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void dock::TrayItem::trayPluginModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::TrayItem::quickPluginModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::TrayItem::fixedPluginModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dock::TrayItem::pluginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
