/****************************************************************************
** Meta object code from reading C++ file 'dockpositioner.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dockpositioner.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dockpositioner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock14DockPositionerE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::DockPositioner::qt_create_metaobjectdata<qt_meta_tag_ZN4dock14DockPositionerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::DockPositioner",
        "QML.Element",
        "DockPositioner",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "DockPositioner is only available via attached properties.",
        "QML.Attached",
        "boundingChanged",
        "",
        "xChanged",
        "yChanged",
        "update",
        "updatePosition",
        "bounding",
        "QRect",
        "x",
        "y"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'boundingChanged'
        QtMocHelpers::SignalData<void()>(8, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xChanged'
        QtMocHelpers::SignalData<void()>(10, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'yChanged'
        QtMocHelpers::SignalData<void()>(11, 9, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(12, 9, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updatePosition'
        QtMocHelpers::SlotData<void()>(13, 9, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'bounding'
        QtMocHelpers::PropertyData<QRect>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'x'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 1),
        // property 'y'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    2 },
    });
    return QtMocHelpers::metaObjectData<DockPositioner, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject dock::DockPositioner::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14DockPositionerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14DockPositionerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock14DockPositionerE_t>.metaTypes,
    nullptr
} };

void dock::DockPositioner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockPositioner *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->boundingChanged(); break;
        case 1: _t->xChanged(); break;
        case 2: _t->yChanged(); break;
        case 3: _t->update(); break;
        case 4: _t->updatePosition(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DockPositioner::*)()>(_a, &DockPositioner::boundingChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPositioner::*)()>(_a, &DockPositioner::xChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPositioner::*)()>(_a, &DockPositioner::yChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QRect*>(_v) = _t->bounding(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->x(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->y(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBounding(*reinterpret_cast<QRect*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dock::DockPositioner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::DockPositioner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14DockPositionerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dock::DockPositioner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void dock::DockPositioner::boundingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::DockPositioner::xChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::DockPositioner::yChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN4dock19DockPanelPositionerE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::DockPanelPositioner::qt_create_metaobjectdata<qt_meta_tag_ZN4dock19DockPanelPositionerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::DockPanelPositioner",
        "QML.Element",
        "DockPanelPositioner",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "DockPanelPositioner is only available via attached properties.",
        "QML.Attached",
        "horizontalOffsetChanged",
        "",
        "vertialOffsetChanged",
        "updatePosition",
        "horizontalOffset",
        "vertialOffset"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'horizontalOffsetChanged'
        QtMocHelpers::SignalData<void()>(8, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'vertialOffsetChanged'
        QtMocHelpers::SignalData<void()>(10, 9, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updatePosition'
        QtMocHelpers::SlotData<void()>(11, 9, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'horizontalOffset'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 0),
        // property 'vertialOffset'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    2 },
    });
    return QtMocHelpers::metaObjectData<DockPanelPositioner, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject dock::DockPanelPositioner::staticMetaObject = { {
    QMetaObject::SuperData::link<DockPositioner::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock19DockPanelPositionerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock19DockPanelPositionerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock19DockPanelPositionerE_t>.metaTypes,
    nullptr
} };

void dock::DockPanelPositioner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DockPanelPositioner *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->horizontalOffsetChanged(); break;
        case 1: _t->vertialOffsetChanged(); break;
        case 2: _t->updatePosition(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DockPanelPositioner::*)()>(_a, &DockPanelPositioner::horizontalOffsetChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DockPanelPositioner::*)()>(_a, &DockPanelPositioner::vertialOffsetChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->horizontalOffset(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->vertialOffset(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHorizontalOffset(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setVertialOffset(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetHorizontalOffset(); break;
        case 1: _t->resetVertialOffset(); break;
        default: break;
        }
    }
}

const QMetaObject *dock::DockPanelPositioner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::DockPanelPositioner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock19DockPanelPositionerE_t>.strings))
        return static_cast<void*>(this);
    return DockPositioner::qt_metacast(_clname);
}

int dock::DockPanelPositioner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DockPositioner::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dock::DockPanelPositioner::horizontalOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::DockPanelPositioner::vertialOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
