/****************************************************************************
** Meta object code from reading C++ file 'dbusadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dbusadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12notification11DbusAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto notification::DbusAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN12notification11DbusAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "notification::DbusAdaptor",
        "D-Bus Interface",
        "org.freedesktop.Notifications",
        "ActionInvoked",
        "",
        "id",
        "actionKey",
        "NotificationClosed",
        "reason",
        "ActivationToken",
        "token",
        "GetCapabilities",
        "Notify",
        "appName",
        "replacesId",
        "appIcon",
        "summary",
        "body",
        "actions",
        "QVariantMap",
        "hints",
        "expireTimeout",
        "CloseNotification",
        "GetServerInformation",
        "QString&",
        "name",
        "vendor",
        "version",
        "specVersion"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ActionInvoked'
        QtMocHelpers::SignalData<void(uint, const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'NotificationClosed'
        QtMocHelpers::SignalData<void(uint, uint)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::UInt, 8 },
        }}),
        // Signal 'ActivationToken'
        QtMocHelpers::SignalData<void(uint, const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'GetCapabilities'
        QtMocHelpers::SlotData<QStringList()>(11, 4, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'Notify'
        QtMocHelpers::SlotData<uint(const QString &, uint, const QString &, const QString &, const QString &, const QStringList &, const QVariantMap &, int)>(12, 4, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::QString, 13 }, { QMetaType::UInt, 14 }, { QMetaType::QString, 15 }, { QMetaType::QString, 16 },
            { QMetaType::QString, 17 }, { QMetaType::QStringList, 18 }, { 0x80000000 | 19, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'CloseNotification'
        QtMocHelpers::SlotData<void(uint)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 },
        }}),
        // Slot 'GetServerInformation'
        QtMocHelpers::SlotData<void(QString &, QString &, QString &, QString &)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 24, 26 }, { 0x80000000 | 24, 27 }, { 0x80000000 | 24, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<DbusAdaptor, qt_meta_tag_ZN12notification11DbusAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject notification::DbusAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification11DbusAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification11DbusAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12notification11DbusAdaptorE_t>.metaTypes,
    nullptr
} };

void notification::DbusAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DbusAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ActionInvoked((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->NotificationClosed((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 2: _t->ActivationToken((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: { QStringList _r = _t->GetCapabilities();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 4: { uint _r = _t->Notify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->CloseNotification((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 6: _t->GetServerInformation((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DbusAdaptor::*)(uint , const QString & )>(_a, &DbusAdaptor::ActionInvoked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DbusAdaptor::*)(uint , uint )>(_a, &DbusAdaptor::NotificationClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DbusAdaptor::*)(uint , const QString & )>(_a, &DbusAdaptor::ActivationToken, 2))
            return;
    }
}

const QMetaObject *notification::DbusAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *notification::DbusAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification11DbusAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int notification::DbusAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void notification::DbusAdaptor::ActionInvoked(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void notification::DbusAdaptor::NotificationClosed(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void notification::DbusAdaptor::ActivationToken(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto notification::DDENotificationDbusAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "notification::DDENotificationDbusAdaptor",
        "D-Bus Interface",
        "org.deepin.dde.Notification1",
        "ActionInvoked",
        "",
        "id",
        "actionKey",
        "NotificationClosed",
        "reason",
        "ActivationToken",
        "token",
        "AppAdded",
        "appId",
        "AppRemoved",
        "AppInfoChanged",
        "configItem",
        "QDBusVariant",
        "value",
        "AppSettingChanged",
        "settings",
        "SystemSettingChanged",
        "SystemInfoChanged",
        "NotificationStateChanged",
        "processedType",
        "RecordCountChanged",
        "count",
        "GetCapabilities",
        "Notify",
        "appName",
        "replacesId",
        "appIcon",
        "summary",
        "body",
        "actions",
        "QVariantMap",
        "hints",
        "expireTimeout",
        "CloseNotification",
        "GetServerInformation",
        "QString&",
        "name",
        "vendor",
        "version",
        "specVersion",
        "recordCount",
        "GetAppList",
        "GetAppInfo",
        "SetAppInfo",
        "GetAppSetting",
        "SetAppSetting",
        "SetSystemInfo",
        "GetSystemInfo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ActionInvoked'
        QtMocHelpers::SignalData<void(uint, const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'NotificationClosed'
        QtMocHelpers::SignalData<void(uint, uint)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::UInt, 8 },
        }}),
        // Signal 'ActivationToken'
        QtMocHelpers::SignalData<void(uint, const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'AppAdded'
        QtMocHelpers::SignalData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'AppRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'AppInfoChanged'
        QtMocHelpers::SignalData<void(const QString &, uint, const QDBusVariant &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::UInt, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Signal 'AppSettingChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'SystemSettingChanged'
        QtMocHelpers::SignalData<void(const QString &)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'SystemInfoChanged'
        QtMocHelpers::SignalData<void(uint, const QDBusVariant &)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Signal 'NotificationStateChanged'
        QtMocHelpers::SignalData<void(qint64, int)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 5 }, { QMetaType::Int, 23 },
        }}),
        // Signal 'RecordCountChanged'
        QtMocHelpers::SignalData<void(uint)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 25 },
        }}),
        // Slot 'GetCapabilities'
        QtMocHelpers::SlotData<QStringList()>(26, 4, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'Notify'
        QtMocHelpers::SlotData<uint(const QString &, uint, const QString &, const QString &, const QString &, const QStringList &, const QVariantMap &, int)>(27, 4, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::QString, 28 }, { QMetaType::UInt, 29 }, { QMetaType::QString, 30 }, { QMetaType::QString, 31 },
            { QMetaType::QString, 32 }, { QMetaType::QStringList, 33 }, { 0x80000000 | 34, 35 }, { QMetaType::Int, 36 },
        }}),
        // Slot 'CloseNotification'
        QtMocHelpers::SlotData<void(uint)>(37, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 5 },
        }}),
        // Slot 'GetServerInformation'
        QtMocHelpers::SlotData<void(QString &, QString &, QString &, QString &)>(38, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 39, 41 }, { 0x80000000 | 39, 42 }, { 0x80000000 | 39, 43 },
        }}),
        // Slot 'recordCount'
        QtMocHelpers::SlotData<uint() const>(44, 4, QMC::AccessPublic, QMetaType::UInt),
        // Slot 'GetAppList'
        QtMocHelpers::SlotData<QStringList()>(45, 4, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'GetAppInfo'
        QtMocHelpers::SlotData<QDBusVariant(const QString &, uint)>(46, 4, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 12 }, { QMetaType::UInt, 15 },
        }}),
        // Slot 'SetAppInfo'
        QtMocHelpers::SlotData<void(const QString &, uint, const QDBusVariant &)>(47, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::UInt, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'GetAppSetting'
        QtMocHelpers::SlotData<QString(const QString &)>(48, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'SetAppSetting'
        QtMocHelpers::SlotData<void(const QString &)>(49, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'SetSystemInfo'
        QtMocHelpers::SlotData<void(uint, const QDBusVariant &)>(50, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'GetSystemInfo'
        QtMocHelpers::SlotData<QDBusVariant(uint)>(51, 4, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::UInt, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'recordCount'
        QtMocHelpers::PropertyData<uint>(44, QMetaType::UInt, QMC::DefaultPropertyFlags, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<DDENotificationDbusAdaptor, qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject notification::DDENotificationDbusAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>.metaTypes,
    nullptr
} };

void notification::DDENotificationDbusAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DDENotificationDbusAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ActionInvoked((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->NotificationClosed((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 2: _t->ActivationToken((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->AppAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->AppRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->AppInfoChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[3]))); break;
        case 6: _t->AppSettingChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->SystemSettingChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->SystemInfoChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2]))); break;
        case 9: _t->NotificationStateChanged((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->RecordCountChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 11: { QStringList _r = _t->GetCapabilities();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 12: { uint _r = _t->Notify((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->CloseNotification((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 14: _t->GetServerInformation((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[4]))); break;
        case 15: { uint _r = _t->recordCount();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 16: { QStringList _r = _t->GetAppList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusVariant _r = _t->GetAppInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->SetAppInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[3]))); break;
        case 19: { QString _r = _t->GetAppSetting((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->SetAppSetting((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->SetSystemInfo((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2]))); break;
        case 22: { QDBusVariant _r = _t->GetSystemInfo((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(uint , const QString & )>(_a, &DDENotificationDbusAdaptor::ActionInvoked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(uint , uint )>(_a, &DDENotificationDbusAdaptor::NotificationClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(uint , const QString & )>(_a, &DDENotificationDbusAdaptor::ActivationToken, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(const QString & )>(_a, &DDENotificationDbusAdaptor::AppAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(const QString & )>(_a, &DDENotificationDbusAdaptor::AppRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(const QString & , uint , const QDBusVariant & )>(_a, &DDENotificationDbusAdaptor::AppInfoChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(const QString & )>(_a, &DDENotificationDbusAdaptor::AppSettingChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(const QString & )>(_a, &DDENotificationDbusAdaptor::SystemSettingChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(uint , const QDBusVariant & )>(_a, &DDENotificationDbusAdaptor::SystemInfoChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(qint64 , int )>(_a, &DDENotificationDbusAdaptor::NotificationStateChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DDENotificationDbusAdaptor::*)(uint )>(_a, &DDENotificationDbusAdaptor::RecordCountChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->recordCount(); break;
        default: break;
        }
    }
}

const QMetaObject *notification::DDENotificationDbusAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *notification::DDENotificationDbusAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notification26DDENotificationDbusAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int notification::DDENotificationDbusAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void notification::DDENotificationDbusAdaptor::ActionInvoked(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void notification::DDENotificationDbusAdaptor::NotificationClosed(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void notification::DDENotificationDbusAdaptor::ActivationToken(uint _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void notification::DDENotificationDbusAdaptor::AppAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void notification::DDENotificationDbusAdaptor::AppRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void notification::DDENotificationDbusAdaptor::AppInfoChanged(const QString & _t1, uint _t2, const QDBusVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void notification::DDENotificationDbusAdaptor::AppSettingChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void notification::DDENotificationDbusAdaptor::SystemSettingChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void notification::DDENotificationDbusAdaptor::SystemInfoChanged(uint _t1, const QDBusVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void notification::DDENotificationDbusAdaptor::NotificationStateChanged(qint64 _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void notification::DDENotificationDbusAdaptor::RecordCountChanged(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
