/****************************************************************************
** Meta object code from reading C++ file 'textcalculator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../textcalculator.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textcalculator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TextCalculatorAttached::qt_create_metaobjectdata<qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TextCalculatorAttached",
        "QML.Element",
        "auto",
        "textChanged",
        "",
        "elidedTextChanged",
        "ellipsisWidthChanged",
        "calculatorChanged",
        "updateElidedText",
        "text",
        "elidedText",
        "ellipsisWidth",
        "calculator",
        "TextCalculator*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'elidedTextChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ellipsisWidthChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'calculatorChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateElidedText'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'text'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'elidedText'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'ellipsisWidth'
        QtMocHelpers::PropertyData<qreal>(11, QMetaType::QReal, QMC::DefaultPropertyFlags, 2),
        // property 'calculator'
        QtMocHelpers::PropertyData<TextCalculator*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<TextCalculatorAttached, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject dock::TextCalculatorAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t>.metaTypes,
    nullptr
} };

void dock::TextCalculatorAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextCalculatorAttached *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textChanged(); break;
        case 1: _t->elidedTextChanged(); break;
        case 2: _t->ellipsisWidthChanged(); break;
        case 3: _t->calculatorChanged(); break;
        case 4: _t->updateElidedText(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextCalculatorAttached::*)()>(_a, &TextCalculatorAttached::textChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculatorAttached::*)()>(_a, &TextCalculatorAttached::elidedTextChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculatorAttached::*)()>(_a, &TextCalculatorAttached::ellipsisWidthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculatorAttached::*)()>(_a, &TextCalculatorAttached::calculatorChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TextCalculator* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->elidedText(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->ellipsisWidth(); break;
        case 3: *reinterpret_cast<TextCalculator**>(_v) = _t->calculator(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setText(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dock::TextCalculatorAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TextCalculatorAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22TextCalculatorAttachedE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dock::TextCalculatorAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void dock::TextCalculatorAttached::textChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::TextCalculatorAttached::elidedTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::TextCalculatorAttached::ellipsisWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dock::TextCalculatorAttached::calculatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN4dock14TextCalculatorE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TextCalculator::qt_create_metaobjectdata<qt_meta_tag_ZN4dock14TextCalculatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TextCalculator",
        "QML.Element",
        "auto",
        "QML.Attached",
        "TextCalculatorAttached",
        "optimalSingleTextWidthChanged",
        "",
        "totalWidthChanged",
        "fontChanged",
        "iconSizeChanged",
        "cellSizeChanged",
        "spacingChanged",
        "itemPaddingChanged",
        "dataModelChanged",
        "remainingSpaceChanged",
        "enabledChanged",
        "onDataModelChanged",
        "onDataModelDestroyed",
        "calculateOptimalTextWidth",
        "optimalSingleTextWidth",
        "totalWidth",
        "dataModel",
        "QAbstractItemModel*",
        "enabled",
        "remainingSpace",
        "font",
        "QFont",
        "iconSize",
        "cellSize",
        "spacing",
        "itemPadding"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'optimalSingleTextWidthChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'totalWidthChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconSizeChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cellSizeChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spacingChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemPaddingChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataModelChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remainingSpaceChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDataModelChanged'
        QtMocHelpers::SlotData<void()>(16, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDataModelDestroyed'
        QtMocHelpers::SlotData<void()>(17, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'calculateOptimalTextWidth'
        QtMocHelpers::SlotData<void()>(18, 6, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'optimalSingleTextWidth'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags, 0),
        // property 'totalWidth'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags, 1),
        // property 'dataModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'remainingSpace'
        QtMocHelpers::PropertyData<qreal>(24, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'font'
        QtMocHelpers::PropertyData<QFont>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'iconSize'
        QtMocHelpers::PropertyData<qreal>(27, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'cellSize'
        QtMocHelpers::PropertyData<qreal>(28, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'spacing'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'itemPadding'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<TextCalculator, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject dock::TextCalculator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14TextCalculatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14TextCalculatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock14TextCalculatorE_t>.metaTypes,
    nullptr
} };

void dock::TextCalculator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextCalculator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->optimalSingleTextWidthChanged(); break;
        case 1: _t->totalWidthChanged(); break;
        case 2: _t->fontChanged(); break;
        case 3: _t->iconSizeChanged(); break;
        case 4: _t->cellSizeChanged(); break;
        case 5: _t->spacingChanged(); break;
        case 6: _t->itemPaddingChanged(); break;
        case 7: _t->dataModelChanged(); break;
        case 8: _t->remainingSpaceChanged(); break;
        case 9: _t->enabledChanged(); break;
        case 10: _t->onDataModelChanged(); break;
        case 11: _t->onDataModelDestroyed(); break;
        case 12: _t->calculateOptimalTextWidth(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::optimalSingleTextWidthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::totalWidthChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::fontChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::iconSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::cellSizeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::spacingChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::itemPaddingChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::dataModelChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::remainingSpaceChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextCalculator::*)()>(_a, &TextCalculator::enabledChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->optimalSingleTextWidth(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->totalWidth(); break;
        case 2: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->dataModel(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->remainingSpace(); break;
        case 5: *reinterpret_cast<QFont*>(_v) = _t->font(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->iconSize(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->cellSize(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->spacing(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->itemPadding(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setDataModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 3: _t->setEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setRemainingSpace(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setFont(*reinterpret_cast<QFont*>(_v)); break;
        case 6: _t->setIconSize(*reinterpret_cast<qreal*>(_v)); break;
        case 7: _t->setCellSize(*reinterpret_cast<qreal*>(_v)); break;
        case 8: _t->setSpacing(*reinterpret_cast<int*>(_v)); break;
        case 9: _t->setItemPadding(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dock::TextCalculator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TextCalculator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock14TextCalculatorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int dock::TextCalculator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void dock::TextCalculator::optimalSingleTextWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::TextCalculator::totalWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::TextCalculator::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dock::TextCalculator::iconSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dock::TextCalculator::cellSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dock::TextCalculator::spacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void dock::TextCalculator::itemPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void dock::TextCalculator::dataModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dock::TextCalculator::remainingSpaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dock::TextCalculator::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
