/****************************************************************************
** Meta object code from reading C++ file 'dsqmlglobal_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../private/dsqmlglobal_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dsqmlglobal_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN2ds10DQmlGlobalE_t {};
} // unnamed namespace

template <> constexpr inline auto ds::DQmlGlobal::qt_create_metaobjectdata<qt_meta_tag_ZN2ds10DQmlGlobalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ds::DQmlGlobal",
        "rootAppletChanged",
        "",
        "applet",
        "DAppletProxy*",
        "pluginId",
        "appletList",
        "QList<DAppletProxy*>",
        "closeChildrenWindows",
        "QWindow*",
        "target",
        "grabKeyboard",
        "grab",
        "grabMouse",
        "singleShot",
        "msec",
        "QJSValue",
        "callback",
        "rootApplet",
        "DApplet*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rootAppletChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'applet'
        QtMocHelpers::MethodData<DAppletProxy *(const QString &) const>(3, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 5 },
        }}),
        // Method 'appletList'
        QtMocHelpers::MethodData<QList<DAppletProxy*>(const QString &) const>(6, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 5 },
        }}),
        // Method 'closeChildrenWindows'
        QtMocHelpers::MethodData<void(QWindow *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'grabKeyboard'
        QtMocHelpers::MethodData<bool(QWindow *, bool)>(11, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 12 },
        }}),
        // Method 'grabKeyboard'
        QtMocHelpers::MethodData<bool(QWindow *)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'grabMouse'
        QtMocHelpers::MethodData<bool(QWindow *, bool)>(13, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 12 },
        }}),
        // Method 'grabMouse'
        QtMocHelpers::MethodData<bool(QWindow *)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'singleShot'
        QtMocHelpers::MethodData<void(int, QJSValue)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rootApplet'
        QtMocHelpers::PropertyData<DApplet*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DQmlGlobal, qt_meta_tag_ZN2ds10DQmlGlobalE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ds::DQmlGlobal::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2ds10DQmlGlobalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2ds10DQmlGlobalE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN2ds10DQmlGlobalE_t>.metaTypes,
    nullptr
} };

void ds::DQmlGlobal::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DQmlGlobal *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rootAppletChanged(); break;
        case 1: { DAppletProxy* _r = _t->applet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DAppletProxy**>(_a[0]) = std::move(_r); }  break;
        case 2: { QList<DAppletProxy*> _r = _t->appletList((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<DAppletProxy*>*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->closeChildrenWindows((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1]))); break;
        case 4: { bool _r = _t->grabKeyboard((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->grabKeyboard((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->grabMouse((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->grabMouse((*reinterpret_cast<std::add_pointer_t<QWindow*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->singleShot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DQmlGlobal::*)()>(_a, &DQmlGlobal::rootAppletChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<DApplet**>(_v) = _t->rootApplet(); break;
        default: break;
        }
    }
}

const QMetaObject *ds::DQmlGlobal::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ds::DQmlGlobal::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN2ds10DQmlGlobalE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DObject"))
        return static_cast< Dtk::Core::DObject*>(this);
    return QObject::qt_metacast(_clname);
}

int ds::DQmlGlobal::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ds::DQmlGlobal::rootAppletChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
