/****************************************************************************
** Meta object code from reading C++ file 'treelandwindowmonitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../treelandwindowmonitor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treelandwindowmonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::ForeignToplevelManager::qt_create_metaobjectdata<qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::ForeignToplevelManager",
        "newForeignToplevelHandle",
        "",
        "ForeignToplevelHandle*",
        "handle"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newForeignToplevelHandle'
        QtMocHelpers::SignalData<void(ForeignToplevelHandle *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ForeignToplevelManager, qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::ForeignToplevelManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<ForeignToplevelManager>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>.metaTypes,
    nullptr
} };

void dock::ForeignToplevelManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ForeignToplevelManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newForeignToplevelHandle((*reinterpret_cast<std::add_pointer_t<ForeignToplevelHandle*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ForeignToplevelHandle* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelManager::*)(ForeignToplevelHandle * )>(_a, &ForeignToplevelManager::newForeignToplevelHandle, 0))
            return;
    }
}

const QMetaObject *dock::ForeignToplevelManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::ForeignToplevelManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock22ForeignToplevelManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_foreign_toplevel_manager_v1"))
        return static_cast< QtWayland::treeland_foreign_toplevel_manager_v1*>(this);
    return QWaylandClientExtensionTemplate<ForeignToplevelManager>::qt_metacast(_clname);
}

int dock::ForeignToplevelManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<ForeignToplevelManager>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void dock::ForeignToplevelManager::newForeignToplevelHandle(ForeignToplevelHandle * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TreeLandDockPreviewContext::qt_create_metaobjectdata<qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TreeLandDockPreviewContext",
        "entered",
        "",
        "exited",
        "closed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entered'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exited'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeLandDockPreviewContext, qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::TreeLandDockPreviewContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<TreeLandDockPreviewContext>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>.metaTypes,
    nullptr
} };

void dock::TreeLandDockPreviewContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeLandDockPreviewContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entered(); break;
        case 1: _t->exited(); break;
        case 2: _t->closed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreeLandDockPreviewContext::*)()>(_a, &TreeLandDockPreviewContext::entered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeLandDockPreviewContext::*)()>(_a, &TreeLandDockPreviewContext::exited, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeLandDockPreviewContext::*)()>(_a, &TreeLandDockPreviewContext::closed, 2))
            return;
    }
}

const QMetaObject *dock::TreeLandDockPreviewContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TreeLandDockPreviewContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock26TreeLandDockPreviewContextE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_dock_preview_context_v1"))
        return static_cast< QtWayland::treeland_dock_preview_context_v1*>(this);
    return QWaylandClientExtensionTemplate<TreeLandDockPreviewContext>::qt_metacast(_clname);
}

int dock::TreeLandDockPreviewContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<TreeLandDockPreviewContext>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void dock::TreeLandDockPreviewContext::entered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dock::TreeLandDockPreviewContext::exited()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dock::TreeLandDockPreviewContext::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto dock::TreeLandWindowMonitor::qt_create_metaobjectdata<qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dock::TreeLandWindowMonitor",
        "handleForeignToplevelHandleAdded",
        "",
        "handleForeignToplevelHandleRemoved",
        "updateFullscreenState"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'handleForeignToplevelHandleAdded'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleForeignToplevelHandleRemoved'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateFullscreenState'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeLandWindowMonitor, qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dock::TreeLandWindowMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractWindowMonitor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>.metaTypes,
    nullptr
} };

void dock::TreeLandWindowMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeLandWindowMonitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->handleForeignToplevelHandleAdded(); break;
        case 1: _t->handleForeignToplevelHandleRemoved(); break;
        case 2: _t->updateFullscreenState(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *dock::TreeLandWindowMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dock::TreeLandWindowMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4dock21TreeLandWindowMonitorE_t>.strings))
        return static_cast<void*>(this);
    return AbstractWindowMonitor::qt_metacast(_clname);
}

int dock::TreeLandWindowMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractWindowMonitor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
