/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.ds.dock.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "dockadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DockAdaptor
 */

DockAdaptor::DockAdaptor(dock::DockPanel *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DockAdaptor::~DockAdaptor()
{
    // destructor
}

QRect DockAdaptor::geometry() const
{
    // get the value of property geometry
    return qvariant_cast< QRect >(parent()->property("geometry"));
}

int DockAdaptor::position() const
{
    // get the value of property position
    return qvariant_cast< int >(parent()->property("position"));
}

void DockAdaptor::setPosition(int value)
{
    // set the value of property position
    parent()->setProperty("position", QVariant::fromValue(value));
}

bool DockAdaptor::showInPrimary() const
{
    // get the value of property showInPrimary
    return qvariant_cast< bool >(parent()->property("showInPrimary"));
}

void DockAdaptor::setShowInPrimary(bool value)
{
    // set the value of property showInPrimary
    parent()->setProperty("showInPrimary", QVariant::fromValue(value));
}

void DockAdaptor::ReloadPlugins()
{
    // handle method call org.deepin.dde.Dock1.ReloadPlugins
    parent()->ReloadPlugins();
}

void DockAdaptor::callShow()
{
    // handle method call org.deepin.dde.Dock1.callShow
    parent()->callShow();
}


#include "moc_dockadaptor.cpp"
