/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.ds.dock.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DOCKADAPTOR_H
#define DOCKADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dockpanel.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.deepin.dde.Dock1
 */
class DockAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.deepin.dde.Dock1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.deepin.dde.Dock1\">\n"
"    <property access=\"read\" type=\"(iiii)\" name=\"geometry\">\n"
"      <annotation value=\"QRect\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"    </property>\n"
"    <property access=\"readwrite\" type=\"i\" name=\"position\"/>\n"
"    <property access=\"readwrite\" type=\"b\" name=\"showInPrimary\"/>\n"
"    <method name=\"callShow\"/>\n"
"    <method name=\"ReloadPlugins\"/>\n"
"  </interface>\n"
        "")
public:
    DockAdaptor(dock::DockPanel *parent);
    ~DockAdaptor() override;

    inline dock::DockPanel *parent() const
    { return static_cast<dock::DockPanel *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QRect geometry READ geometry)
    QRect geometry() const;

    Q_PROPERTY(int position READ position WRITE setPosition)
    int position() const;
    void setPosition(int value);

    Q_PROPERTY(bool showInPrimary READ showInPrimary WRITE setShowInPrimary)
    bool showInPrimary() const;
    void setShowInPrimary(bool value);

public Q_SLOTS: // METHODS
    void ReloadPlugins();
    void callShow();
Q_SIGNALS: // SIGNALS
};

#endif
