/****************************************************************************
** Meta object code from reading C++ file 'notifyaccessor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../notifyaccessor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notifyaccessor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t {};
} // unnamed namespace

template <> constexpr inline auto notifycenter::NotifyAccessor::qt_create_metaobjectdata<qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "notifycenter::NotifyAccessor",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "entityReceived",
        "",
        "id",
        "stagingEntityReceived",
        "stagingEntityClosed",
        "debuggingChanged",
        "enabledChanged",
        "dataUpdaterChanged",
        "onReceivedRecord",
        "openNotificationSetting",
        "onNotificationStateChanged",
        "processedType",
        "debugging",
        "enabled",
        "dataUpdater"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entityReceived'
        QtMocHelpers::SignalData<void(qint64)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 },
        }}),
        // Signal 'stagingEntityReceived'
        QtMocHelpers::SignalData<void(qint64)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 },
        }}),
        // Signal 'stagingEntityClosed'
        QtMocHelpers::SignalData<void(qint64)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 },
        }}),
        // Signal 'debuggingChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataUpdaterChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onReceivedRecord'
        QtMocHelpers::SlotData<void(const QString &)>(13, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'openNotificationSetting'
        QtMocHelpers::MethodData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'onNotificationStateChanged'
        QtMocHelpers::MethodData<void(qint64, int)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 }, { QMetaType::Int, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'debugging'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'dataUpdater'
        QtMocHelpers::PropertyData<QObject*>(19, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<NotifyAccessor, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject notifycenter::NotifyAccessor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t>.metaTypes,
    nullptr
} };

void notifycenter::NotifyAccessor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NotifyAccessor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entityReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 1: _t->stagingEntityReceived((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 2: _t->stagingEntityClosed((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 3: _t->debuggingChanged(); break;
        case 4: _t->enabledChanged(); break;
        case 5: _t->dataUpdaterChanged(); break;
        case 6: _t->onReceivedRecord((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->openNotificationSetting(); break;
        case 8: _t->onNotificationStateChanged((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)(qint64 )>(_a, &NotifyAccessor::entityReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)(qint64 )>(_a, &NotifyAccessor::stagingEntityReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)(qint64 )>(_a, &NotifyAccessor::stagingEntityClosed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)()>(_a, &NotifyAccessor::debuggingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)()>(_a, &NotifyAccessor::enabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NotifyAccessor::*)()>(_a, &NotifyAccessor::dataUpdaterChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->debugging(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 2: *reinterpret_cast<QObject**>(_v) = _t->dataUpdater(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setDataUpdater(*reinterpret_cast<QObject**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *notifycenter::NotifyAccessor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *notifycenter::NotifyAccessor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12notifycenter14NotifyAccessorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int notifycenter::NotifyAccessor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void notifycenter::NotifyAccessor::entityReceived(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void notifycenter::NotifyAccessor::stagingEntityReceived(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void notifycenter::NotifyAccessor::stagingEntityClosed(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void notifycenter::NotifyAccessor::debuggingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void notifycenter::NotifyAccessor::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void notifycenter::NotifyAccessor::dataUpdaterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
