/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.ds.dock.taskmanager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef TASKMANAGERADAPTOR_H
#define TASKMANAGERADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "taskmanager.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.deepin.ds.Dock.TaskManager
 */
class TaskManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.deepin.ds.Dock.TaskManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.deepin.ds.Dock.TaskManager\">\n"
"    <property access=\"readwrite\" type=\"b\" name=\"windowSplit\"/>\n"
"    <property access=\"readwrite\" type=\"b\" name=\"allowForceQuit\"/>\n"
"  </interface>\n"
        "")
public:
    TaskManagerAdaptor(dock::TaskManager *parent);
    ~TaskManagerAdaptor() override;

    inline dock::TaskManager *parent() const
    { return static_cast<dock::TaskManager *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(bool allowForceQuit READ allowForceQuit WRITE setAllowForceQuit)
    bool allowForceQuit() const;
    void setAllowForceQuit(bool value);

    Q_PROPERTY(bool windowSplit READ windowSplit WRITE setWindowSplit)
    bool windowSplit() const;
    void setWindowSplit(bool value);

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
};

#endif
